using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using Components.Agenda.GUI;
using System.Drawing;
using Utils.Misc;
using System.Runtime.Serialization;
using System.Drawing.Drawing2D;
using Components.Agenda.Management;
using System.ComponentModel;

namespace Components.Agenda.Management
{
    #region MONTH
    /// <summary>
    /// Structure contenant les informations d'un rendez-vous
    /// </summary>
    [Serializable]
    public struct MeetingRow : IComparable<MeetingRow>
    {
        #region Variables
        /// <summary>
        /// Titre du rendez-vous
        /// </summary>        
        private string vTitle;
        /// <summary>
        /// Message du rendez-vous
        /// </summary>
        private string vMessage;
        /// <summary>
        /// Date de dbut du rendez-vous
        /// </summary>
        private DateTime vStartDate;
        /// <summary>
        /// Date de fin du rendez-vous
        /// </summary>
        private DateTime vEndDate;
        /// <summary>
        /// Identifiant du client
        /// </summary>
        private int vIDPatient;
        /// <summary>
        /// Identifiant du rendez-vous
        /// </summary>
        private int vIDMeeting;
        /// <summary>
        /// Prix d'un honoraire
        /// </summary>
        private int vHonoraire;
        #endregion

        #region Mthodes
        /// <summary>
        /// Compare l'instance en cours  un objet du mme type.
        /// </summary>
        /// <param name="other">MeetingRow  comparer  cette instance.</param>
        /// <returns> Entier sign 32bits qui indique l'ordre relatif des objets compars. La
        /// valeur de retour a les significations suivantes: Valeur Signification Infrieure
        ///  zro Cette instance est infrieure  obj. Zro Cette instance est gale
        ///  obj. Suprieure  zro Cette instance est suprieure  obj.</returns> 
        ///<exception cref="ArgumentException">obj n'est pas du mme type que cette instance.</exception>
        public int CompareTo(MeetingRow other)
        {
            if (other.StartDate > this.StartDate)
                return -1;
            else if (other.StartDate < this.StartDate)
                return 1;
            else
                return 0;
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise le titre du rendez-vous
        /// </summary>                
        public string Title { get { return vTitle; } set { vTitle = value; } }

        /// <summary>
        /// Retourne ou initialise le message du rendez-vous
        /// </summary>        
        public string Message { get { return vMessage; } set { vMessage = value; } }

        /// <summary>
        /// Retourne ou initialise la date de dbut du rendez-vous
        /// </summary>
        public DateTime StartDate { get { return vStartDate; } set { vStartDate = value; } }

        /// <summary>
        /// Retourne ou initialise la date de fin du rendez-vous
        /// </summary>
        public DateTime EndDate { get { return vEndDate; } set { vEndDate = value; } }

        /// <summary>
        /// Retourne ou initialise l'identifiant du client
        /// </summary>
        public int IDPatient { get { return vIDPatient; } set { vIDPatient = value; } }

        /// <summary>
        /// Retourne ou initialise l'identifiant du rendez-vous
        /// </summary>
        public int IDMeeting { get { return vIDMeeting; } set { vIDMeeting = value; } }

        /// <summary>
        /// Retourne ou initialise le prix d'un honoraire
        /// </summary>
        public int Honoraire { get { return vHonoraire; } set { vHonoraire = value; } }
        #endregion

        #region Overriden
        /// <summary>
        /// Retourne un rendez-vous sous forme d'un string
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return TextManager.FirstUpper(this.StartDate.ToLongDateString()) + " de " + StartDate.ToShortTimeString()
                + "  " + EndDate.ToShortTimeString();
        }
        #endregion
    }

    /// <summary>
    /// MeetingRow rsultant d'une recherche
    /// </summary>
    public struct ResultMeetingRow
    {
        /// <summary>
        /// Rsultat de la recherche
        /// </summary>
        public MeetingRow Result;
        /// <summary>
        /// True si le MeetingRow est le rsultat ou false si la recherche n'a rien donner
        /// </summary>
        public bool IsResult;
    }

    /// <summary>
    /// Classe de base pour la gestion de la liste des rendez-vous
    /// </summary>
    public abstract class RootMeetingSet : IEnumerator, IEnumerable
    {
        #region Variables
        /// <summary>
        /// Index pour l'numration
        /// </summary>
        private int Index = -1;
        /// <summary>
        /// Liste de tous les rendez-vous
        /// </summary>
        protected List<MeetingRow> vMeetingList = new List<MeetingRow>();
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public RootMeetingSet() { }

        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_MeetingList">Liste des rendez-vous pour une priode donne</param>
        public RootMeetingSet(List<MeetingRow> _MeetingList) { vMeetingList = _MeetingList; }
        #endregion

        #region Mthodes abstraites
        /// <summary>
        /// Mthode de remplissage de la liste de rendez-vous
        /// </summary>
        public abstract void Fill();
        #endregion

        #region Mthodes
        /// <summary>
        /// Recherche un rendez-vous unique sur base de son identifiant
        /// </summary>
        /// <param name="_IDMeeting">Identifiant du rendez-vous</param>
        /// <returns>Les informations sur le rendez-vous ou null si rien n'a t trouv</returns>
        public ResultMeetingRow Seek(int _IDMeeting)
        {
            ResultMeetingRow result = new ResultMeetingRow();
            result.IsResult = false;

            foreach (MeetingRow mr in vMeetingList)
            {
                if (mr.IDMeeting == _IDMeeting)
                {
                    result.Result = mr;
                    result.IsResult = true;
                    return result;
                }
            }
            return result;

        }
        /// <summary>
        /// Fait une recherche du nombre de rendez-vous  la date donne
        /// </summary>
        /// <param name="_Year">Anne du rendez-vous</param>
        /// <param name="_Month">Mois du rendez-vous</param>
        /// <returns>Une liste de MeetingRow contenant les rendez-vous</returns>
        public List<MeetingRow> Seek(int _Year, int _Month)
        {
            if (vMeetingList == null) return new List<MeetingRow>();

            List<MeetingRow> mr = new List<MeetingRow>();

            foreach (MeetingRow Row in vMeetingList)
            {
                if (Row.StartDate.Year == _Year)
                {
                    if (Row.StartDate.Month == _Month)
                        mr.Add(Row);
                }
            }
            return mr;
        }
        /// <summary>
        /// Fait une recherche du nombre de rendez-vous  la date donne
        /// </summary>
        /// <param name="_Year">Anne du rendez-vous</param>
        /// <param name="_Month">Mois du rendez-vous</param>
        /// <param name="_Day">Jour du rendez-vous</param>
        /// <returns>Une liste de MeetingRow contenant les rendez-vous</returns>
        public List<MeetingRow> Seek(int _Year, int _Month, int _Day)
        {
            if (vMeetingList == null) return new List<MeetingRow>();

            List<MeetingRow> mr = new List<MeetingRow>();

            foreach (MeetingRow Row in vMeetingList)
            {
                if (Row.StartDate.Year == _Year)
                {
                    if (Row.StartDate.Month == _Month)
                    {
                        if (Row.StartDate.Day == _Day)
                            mr.Add(Row);
                    }
                }
            }
            return mr;
        }
        /// <summary>
        /// Recherche un rendez-vous  une date prcise (Heure comprise)
        /// </summary>
        /// <param name="_FullDate">Date du rendez-vous</param>
        /// <returns>Une liste de MeetingRow contenant les rendez-vous</returns>
        public List<MeetingRow> Seek(DateTime _FullDate)
        {
            if (vMeetingList == null) return new List<MeetingRow>();

            List<MeetingRow> mr = new List<MeetingRow>();

            foreach (MeetingRow Row in vMeetingList)
            {
                if (Row.StartDate.Year == _FullDate.Year
                    && Row.StartDate.Month == _FullDate.Month
                    && Row.StartDate.Day == _FullDate.Day)
                    mr.Add(Row);
            }
            return mr;
        }
        /// <summary>
        /// Retourne le nombre de rendez-vous  la date donne
        /// </summary>
        /// <param name="_Year">Anne du rendez-vous</param>
        /// <param name="_Month">Mois du rendez-vous</param>
        /// <returns>Le nombre de rendez-vous  cette date</returns>
        public int CountElementAt(int _Year, int _Month)
        {
            int Count = 0;

            foreach (MeetingRow mr in vMeetingList)
            {
                if (mr.StartDate.Year == _Year
                    && mr.StartDate.Month == _Month)
                    Count++;
            }
            return Count;
        }
        /// <summary>
        /// Retourne le nombre de rendez-vous  la date donne
        /// </summary>
        /// <param name="_Year">Anne du rendez-vous</param>
        /// <param name="_Month">Mois du rendez-vous</param>
        /// <param name="_Day">Jours du rendez-vous</param>
        /// <returns>Le nombre de rendez-vous  cette date</returns>
        public int CountElementAt(int _Year, int _Month, int _Day)
        {
            int Count = 0;

            foreach (MeetingRow mr in vMeetingList)
            {
                if (mr.StartDate.Year == _Year
                    && mr.StartDate.Month == _Month
                    && mr.StartDate.Day == _Day)
                    Count++;
            }
            return Count;
        }
        /// <summary>
        /// Ajoute un lment dans le liste des rendez-vous
        /// </summary>
        /// <param name="_Value">Un rendezv-vous</param>
        /// <returns>Un identifiant de rendez-vous (son index dans la liste)</returns>
        /// <remarks>Cet mthode est virtuelle parce que le code d'insertion dans
        /// La base de donne ou tout autre systme de sauvegarde sur fichier
        /// doit tre implment dans la classe fille</remarks>
        public virtual void Add(MeetingRow _Value)
        {
            vMeetingList.Add(_Value);
            this.Sort();
            //return vMeetingList.IndexOf(_Value);
        }
        /// <summary>
        /// Supprime un rendez-vous  la date donne dans le liste des rendez-vous
        /// </summary>
        /// <param name="_Value">Le rendez-vous  supprimer</param>
        /// <remarks>N'ffectue que une seule suppression</remarks>
        public virtual void Remove(MeetingRow _Value)
        {
            foreach (MeetingRow row in vMeetingList)
            {
                if (row.IDMeeting == _Value.IDMeeting)
                {
                    vMeetingList.Remove(row);
                    break;
                }
            }
        }
        /// <summary>
        /// Fait une modification du rendez-vous dsign par _mr.IDMeeting
        /// </summary>
        /// <param name="_mr">MeetingRow contenant les informations du nouveau rendez-vous</param>        
        public virtual void Modify(MeetingRow _mr)
        {
            for (int i = 0; i < vMeetingList.Count; i++)
            {
                if (vMeetingList[i].IDMeeting == _mr.IDMeeting)
                {
                    vMeetingList[i] = _mr;
                    break;
                }
            }
        }
        /// <summary>
        /// Mthode de triage des rendez-vous par date en ordre croissant
        /// </summary>
        /// <remarks>Utilisation du Bubble Sort</remarks>
        public void Sort()
        {
            int elm = vMeetingList.Count;
            MeetingRow buf;

            for (int i = (elm - 1); i >= 1; i--)
            {
                for (int j = i; j <= i; j++)
                {
                    if (vMeetingList[j - 1].StartDate > vMeetingList[j].StartDate)
                    {
                        buf = vMeetingList[j - 1];
                        vMeetingList[j - 1] = vMeetingList[j];
                        vMeetingList[j] = buf;
                    }
                }
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne l'lment  l'index donn
        /// </summary>
        /// <param name="_index">index qu'il faut retourner</param>
        /// <returns>Le rendez-vous  l'index donn</returns>
        /// <exception cref="ArgumentOutOfRangeException">En cas d'index hors des limites</exception>
        public MeetingRow this[int _index]
        {
            get
            {
                try { return vMeetingList[_index]; }
                catch (ArgumentOutOfRangeException) { throw; }
            }
        }
        /// <summary>
        /// Retourne le nombre de rendez-vous dans la liste
        /// </summary>
        public int Count { get { return vMeetingList.Count; } }
        /// <summary>
        /// Retourne ou initialise une List contenant les rendez-vous
        /// </summary>
        public List<MeetingRow> MeetingList
        {
            get { return this.vMeetingList; }
            set { vMeetingList = value; }
        }
        #endregion

        #region IEnumerator Members
        /// <summary>
        /// Retourne l'lment courrant
        /// </summary>
        public object Current { get { return vMeetingList[Index]; } }

        /// <summary>
        /// Vrifie si il reste un lment suivant
        /// </summary>
        /// <returns></returns>
        public bool MoveNext()
        {
            Index++;
            return (Index < vMeetingList.Count);
        }

        /// <summary>
        /// Rinitialise l'index de la liste
        /// </summary>
        public void Reset() { Index = -1; }

        #endregion

        #region IEnumerable Members
        /// <summary>
        /// Returns an enumerator that iterates through a collection.
        /// </summary> An System.Collections.IEnumerator object that can be used to iterate through
        /// the collection.
        public IEnumerator GetEnumerator() { Index = -1; return (IEnumerator)this; }

        #endregion

        #region Operator
        /// <summary>
        /// Fait une convertion d'un RootMeetingSet vers une liste de rendez-vous
        /// </summary>
        /// <param name="_RootMeetingSet">Le RootMeetingSet  convertir</param>
        /// <returns>Une List(MeetingRow) contenue dans le RootMeetingSet</returns>
        public static implicit operator List<MeetingRow>(RootMeetingSet _RootMeetingSet)
        {
            return _RootMeetingSet.vMeetingList;
        }
        #endregion
    }

    /// <summary>
    /// Classe de gestion des jours du mois pour l'affichage
    /// </summary>
    public class Days : IEnumerator
    {
        #region Variables
        /// <summary>
        /// Variable d'numaration pour this[]
        /// </summary>
        protected int vEnumerator = -1;

        /// <summary>
        /// Tableau de jours 
        /// </summary>
        protected MonthRectangleInfo[] vDays = new MonthRectangleInfo[42];

        /// <summary>
        /// Police du texte de la cellule
        /// </summary>
        protected Font vFont = new Font("Times new Roman", 12);

        /// <summary>
        /// Couleur du texte
        /// </summary>
        protected Color vColorText = Color.Black;

        /// <summary>
        /// Couleur de fond
        /// </summary>
        protected Color vColorBack = Color.White;

        /// <summary>
        /// Couleur des case de jour hors du mois courrant
        /// </summary>
        protected Color vColorNonMonthBack = Color.Silver;

        /// <summary>
        /// Couleur de la case du jour d'aujourd'hui
        /// </summary>
        protected Color vTodayBackColor = Color.Red;

        /// <summary>
        /// Couleur du contour du tableau
        /// </summary>
        protected Pen vSurround = new Pen(Brushes.Black, 1);

        /// <summary>
        /// Point de coord (0,0) pour l'affichage des jours
        /// </summary>
        protected int vCoorY;

        /// <summary>
        /// Taille du PictureBox dans lequel on dessine
        /// </summary>
        protected Size vSizeOfComponent;

        /// <summary>
        /// Date  afficher
        /// </summary>
        protected DateTime vDate = new DateTime(DateTime.Today.Year, DateTime.Today.Month, 1);

        /// <summary>
        /// Liste des rendez-vous du mois courrant
        /// </summary>
        protected List<MeetingRow> vMeetingList;

        ///<summary>
        ///Index de la date d'aujourd'hui
        ///</summary>
        protected int vTodayIndex = -1;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_CoodY">Coordonne Y pour la coord (0,0) du premier point 
        /// (X + hauteur des headers; Y)</param>
        /// <param name="_SizeOfComponent">Taille du PictureBox dans lequel on dessine</param>
        public Days(Size _SizeOfComponent, int _CoodY)
        {
            vCoorY = _CoodY;
            vSizeOfComponent = _SizeOfComponent;

            for (int i = 0; i < vDays.Length; i++)
                vDays[i] = new MonthRectangleInfo();
            Resize();
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Recalcule la taille des cellules des jours
        /// </summary>
        public void Resize()
        {
            SetDate();
            Size SizeCard = new Size(vSizeOfComponent.Width / 7, ((vSizeOfComponent.Height - 1) - vCoorY) / 6);
            int Y = vCoorY;

            Rectangle r = new Rectangle(new Point(0, vCoorY),
                new Size((SizeCard.Width + (vSizeOfComponent.Width % 7)) - 1, SizeCard.Height));

            for (int semaine = 0; semaine < 6; semaine++)
            {
                for (int jour = 0; jour < 7; jour++)
                {
                    if (jour == 0)
                        r = new Rectangle(new Point(r.X, Y),
                            new Size(SizeCard.Width + (vSizeOfComponent.Width % 7) - 1, SizeCard.Height));
                    else
                        r = new Rectangle(new Point(r.X + r.Width, Y), SizeCard);

                    if (vDays[jour + (semaine * 7)] == null)
                        vDays[jour + (semaine * 7)] = new MonthRectangleInfo();

                    vDays[jour + (semaine * 7)].Rectangle = r;
                    vDays[jour + (semaine * 7)].TextColor = vColorText;
                    vDays[jour + (semaine * 7)].TextFont = vFont;
                }
                Y += SizeCard.Height;
                r.X = 0;
            }
        }

        /// <summary>
        /// Recalcule le tableau pour que le mois courrant soit correcte
        /// </summary>
        protected void SetDate()
        {
            DateTime dt = new DateTime(vDate.Year, vDate.Month, 1);
            int FirstDay = (int)dt.DayOfWeek - 1;
            if (FirstDay == -1)
                FirstDay = 6;

            int DaysInMonth = Month.GetDaysInMonth(vDate.Year, vDate.Month) - 1;
            int jour = 0;

            for (int i = 0; i < 42; i++)
            {
                if (vDays[i] == null)
                    vDays[i] = new MonthRectangleInfo();

                //Jours avant le mois
                if (i < FirstDay)
                {
                    vDays[i].DayNumber = "";
                    vDays[i].BackColor = vColorNonMonthBack;
                    vDays[i].Position = CasePositionInMonth.Before;
                }
                //Jour du mois
                else
                {
                    vDays[i].DayNumber = ((i + 1) - FirstDay).ToString();
                    vDays[i].BackColor = vColorBack;
                    vDays[i].Position = CasePositionInMonth.InMonth;

                    if (vMeetingList != null)
                    {
                        int count = 0;
                        foreach (MeetingRow mr in vMeetingList)
                        {
                            if (mr.StartDate.Month == vDate.Month
                                && mr.StartDate.Day == jour + 1)
                            {
                                count++;
                            }
                        }
                        if (count != 0)
                            vDays[i].Text = count + " rdv";
                        jour++;
                    }
                }
                //Jours aprs le mois
                if (i - FirstDay > DaysInMonth)
                {
                    vDays[i].DayNumber = "";
                    vDays[i].BackColor = vColorNonMonthBack;
                    vDays[i].Position = CasePositionInMonth.After;
                }

                //Si nous sommes aujourd'hui
                if (vDate.Month == DateTime.Today.Month
                    && vDate.Year == DateTime.Today.Year && (i - FirstDay) + 1 == DateTime.Today.Day)
                {
                    vDays[i].BackColor = vTodayBackColor;
                    vTodayIndex = i;
                    vDays[i].Position = CasePositionInMonth.InMonth;
                }
                //else
                //    vTodayIndex = -1;
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise la police du texte de la cellule de jour
        /// </summary>
        public Font Font
        {
            get { return vFont; }
            set { vFont = value; }
        }

        /// <summary>
        /// Retourne ou initialise le couleur du texte de la cellule de jour
        /// </summary>
        public Color ColorText
        {
            get { return vColorText; }
            set { vColorText = value; }
        }

        /// <summary>
        /// Retourne ou initialise la couleur de fonc de la cellule de jour
        /// </summary>
        public Color ColorBack
        {
            get { return vColorBack; }
            set { vColorBack = value; }
        }

        /// <summary>
        /// Retourne ou initialise les couleurs des jours hors du mois courrant
        /// </summary>
        public Color ColorNonMonthBack
        {
            get { return vColorNonMonthBack; }
            set { vColorNonMonthBack = value; }
        }

        /// <summary>
        /// Retourne ou initialise la date
        /// </summary>
        public DateTime Date
        {
            get { return vDate; }
            set
            {
                vDate = value;
                SetDate();
            }
        }

        /// <summary>
        /// Accesseurs afin d'atteindre les jours  afficher dans la calendrier
        /// </summary>
        /// <param name="_Index">Index pour accder au jour voulu</param>
        /// <returns>RectangleInfo contenant les information pour l'afichage d'un jour</returns>
        public MonthRectangleInfo this[int _Index]
        {
            get { return vDays[_Index]; }
        }

        /// <summary>
        /// Donne la taille du composant
        /// </summary>
        public Size ComponentSize
        {
            get { return vSizeOfComponent; }
            set
            {
                if (value != vSizeOfComponent)
                {
                    vSizeOfComponent = value;
                    Resize();
                }
            }
        }

        /// <summary>
        /// Retourne le nombre d'lmentss dans la liste des jours du mois
        /// </summary>
        public int Count
        {
            get { return this.vDays.Length; }
        }

        /// <summary>
        /// Retourne ou initialise la couleur des colones du tableau des jours
        /// </summary>
        public Color ColorSurround
        {
            get { return this.vSurround.Color; }
            set
            {
                vSurround = new Pen(value);
                for (int i = 0; i < vDays.Length; i++)
                    vDays[i].BorderPen = new Pen(value);
            }
        }

        /// <summary>
        /// Retourne ou initialise la couleur d'aujourd'hui
        /// </summary>
        public Color TodayBackColor
        {
            get { return vTodayBackColor; }
            set
            {
                this.SetDate();
                vTodayBackColor = value;
                if (this.vTodayIndex != -1)
                    vDays[vTodayIndex].BackColor = value;
            }
        }

        /// <summary>
        /// Retourne l'index du jour d'aujourd'hui ou -1 si ce n'est pas le bon mois ou anne
        /// </summary>
        public int TodayIndex
        { get { return vTodayIndex; } }

        /// <summary>
        /// Retourne true si c'est le mois actuel
        /// </summary>
        public bool IsActualMonth
        {
            get
            {
                if (vTodayIndex != -1) return false;
                else return true;
            }
        }

        /// <summary>
        /// Retourne ou initialise la liste des rendez-vous pour le mois donn
        /// </summary>
        public List<MeetingRow> MeetingList
        {
            get { return vMeetingList; }
            set { vMeetingList = value; }
        }
        #endregion

        #region IEnumerator Membres
        /// <summary>
        /// Retourne une rfrence sur cet objet et met l'index  0
        /// </summary>
        /// <returns>Une rfrence sur this</returns>
        public IEnumerator GetEnumerator() { vEnumerator = -1; return this; }

        /// <summary>
        /// Obtient l'lment en cours dans la collection.
        /// </summary>
        object IEnumerator.Current
        {
            get { return vDays[vEnumerator]; }
        }

        /// <summary>
        /// Avance l'numrateur  l'lment suivant de la collection.       
        /// </summary>
        /// <returns>true si l'numrateur a pu avancer jusqu' l'lment suivant; false si l'numrateur
        /// a dpass la fin de la collection.</returns>
        public bool MoveNext() { return ++vEnumerator < vDays.Length; }

        /// <summary>
        /// Rtablit l'numrateur  sa position initiale, qui prcde le premier lment
        /// de la collection.
        /// </summary>
        public void Reset() { vEnumerator = -1; }
        #endregion
    }
    #endregion

    #region DAY
    /// <summary>
    /// Classe de gestion des affichages des heures d'une journe
    /// </summary>
    public class Hours
    {
        #region Variables
        /// <summary>
        /// Indique le nombre de colonnes pour un moment partag
        /// </summary>
        protected int vColumns = 0;

        #region Const
        /// <summary>
        /// Largeur de la colonne des heures
        /// </summary>
        public const int COLUMN_WIDTH = 50;
        /// <summary>
        /// Hauteur du header
        /// </summary>
        public const int HEADER_HEIGHT = 25;
        /// <summary>
        /// Nombre de partie contenue dans un jour
        /// </summary>
        public const int DAYS_PARTS = 24 * 2;
        #endregion

        #region Options
        /// <summary>
        /// Couleur de fond d'une heure qui n'est pas un rdv
        /// </summary>
        protected Color vNonRdvBackColor = Color.White;
        /// <summary>
        /// Couleur du texte d'une heure qui n'est pas un rdv
        /// </summary>
        protected Color vNonRdvTextColor = Color.Black;
        /// <summary>
        /// Police d'criture d'une heure qui n'est pas un rendez-vous
        /// </summary>
        protected Font vNonRdvTextFont = new Font("Times New Roman", 12);
        /// <summary>
        /// Couleur de fond d'une heure qui est un rendez-vous
        /// </summary>
        protected Color vFirstMeetingFadeColor = Color.Tomato;
        /// <summary>
        /// Deuxime couleur pour le dgrad de fond d'un rendez-vous
        /// </summary>
        protected Color vSecondMeetingFadeColor = Color.Silver;
        /// <summary>
        /// Couleur d'criture d'une heure qui est un rendez-vous
        /// </summary>
        protected Color vRdvTextColor = Color.Black;
        /// <summary>
        /// Police d'criture d'une heure qui est un rendez-vous
        /// </summary>
        protected Font vRdvTextFont = new Font("Times New Roman", 12);
        /// <summary>
        /// Couleur de fond du header
        /// </summary>
        protected Color vHeaderColorBack = Color.White;
        /// <summary>
        /// Police d'criture du header
        /// </summary>
        protected Font vHeaderFont = new Font("Times New Roman", 12);
        /// <summary>
        /// Couleur du texte du header
        /// </summary>
        protected Color vHeaderTextColor = Color.Black;
        #endregion

        #region Misc
        /// <summary>
        /// Contient les entte des heures
        /// </summary>
        protected DayRectangleInfo[] vHourHeader = new DayRectangleInfo[DAYS_PARTS];
        /// <summary>
        /// Liste qui contient tous les rendez-vous du jours courrant
        /// </summary>
        protected List<MeetingRow> vList = new List<MeetingRow>();
        /// <summary>
        /// Taille du composant dans lequel on doit crire
        /// </summary>
        protected Size vSizeOfComponent = new Size(10, 10);
        /// <summary>
        /// Date  afficher
        /// </summary>
        protected DateTime vDate;
        /// <summary>
        /// Liste des rectangles  dessiner
        /// </summary>
        protected List<DayRectangleInfo> vListToDraw;
        #endregion

        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>        
        public Hours()
        {
            vList = new List<MeetingRow>();
            ListToDraw = Recalculate();
        }

        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_List">Initialise la liste de rendez-vous de la journe</param>
        public Hours(List<MeetingRow> _List)
        {
            vList = _List;
            ListToDraw = Recalculate();
        }
        #endregion

        #region Mthodes publique
        /// <summary>
        /// Prpare les rectangles qu'il faudra dessiner dans le PictureBox reprsentant le DayAgenda
        /// </summary>
        /// <returns>Une liste contenant tous les rectangles  dessiner</returns>
        public List<DayRectangleInfo> Recalculate()
        {
            int FirstStep = 0 - Hours.COLUMN_WIDTH;
            //TODO lorsque le rdv suivant ne se chevauche pas. vrifier tous les prcdents en partant du deuxime avant.

            List<DayRectangleInfo> ListToDraw = new List<DayRectangleInfo>();

            #region Il y a des rendez-vous aujourd'hui
            if (vList.Count > 0)
            {
                //Index de la liste des rendez-vous
                int MeetingIdx = 0;

                //Nombre de colonnes
                int Columns = 0;

                //Passage en revue de toutes les parties de la journe
                for (int i = 0; i < DAYS_PARTS; i++)
                {
                    #region C'est un rendez-vous
                    if (MeetingIdx < vList.Count && GetIndex(vList[MeetingIdx].StartDate) == i)
                    {
                        DateTime StartDate = vList[MeetingIdx].StartDate;
                        DateTime EndDate = vList[MeetingIdx].EndDate;
                        Columns = 0;

                        #region Comptage des rendez-vous parallles
                        bool bExit = false;
                        for (int idx = MeetingIdx; idx < vList.Count - 1 && !bExit; idx++)
                        {
                            //Le rendez-vous suivant est en paralle
                            if (vList[idx + 1].StartDate < vList[idx].EndDate)
                            {
                                Columns++;
                                if (EndDate < vList[idx + 1].EndDate)
                                    EndDate = vList[idx + 1].EndDate;
                            }
                            else //Sinon on vrifie que ce rdv ne chevauche aucun rdv prcdent
                            {
                                int ret = 0;
                                for (int prec = idx; prec >= 0; prec--)
                                {
                                    if (vList[prec].EndDate > vList[idx + 1].StartDate)
                                    {
                                        Columns++;
                                        ret++;
                                        if (EndDate < vList[idx + 1].EndDate)
                                            EndDate = vList[idx + 1].EndDate;
                                        break;
                                    }
                                }
                                if (ret == 0)
                                {
                                    bExit = true;
                                    break;
                                }
                            }
                        }
                        #endregion

                        #region Plusieurs colonnes
                        if (++Columns > 1)// && Columns != 0)
                        {
                            //passe en revue les colonnes
                            for (int col = 0; col < Columns; col++, MeetingIdx++)
                            {
                                bool IsTitled = false;

                                //On dessine le rectangle
                                for (int TempIdx = GetIndex(StartDate); TempIdx < GetIndex(EndDate); TempIdx++)
                                {
                                    DayRectangleInfo ri = new DayRectangleInfo();
                                    ri.Rectangle = new Rectangle(
                                        new Point(COLUMN_WIDTH + (col * ((vSizeOfComponent.Width - COLUMN_WIDTH) / Columns)),
                                        TempIdx * HEADER_HEIGHT),
                                        new Size((vSizeOfComponent.Width - COLUMN_WIDTH) / Columns, HEADER_HEIGHT));

                                    //Si index dans le rendez-vous
                                    if (GetIndex(vList[MeetingIdx].StartDate) <= TempIdx
                                        && GetIndex(vList[MeetingIdx].EndDate) > TempIdx)
                                    {
                                        ri.TextFont = vRdvTextFont;
                                        ri.TextColor = vRdvTextColor;
                                        ri.BackColor = vFirstMeetingFadeColor;
                                        ri.BackColor2 = vSecondMeetingFadeColor;
                                        ri.IDMeeting = vList[MeetingIdx].IDMeeting;

                                        //Est-ce un dbut de rendez-vous?
                                        if (!IsTitled)
                                        {
                                            ri.Text = vList[MeetingIdx].Title;
                                            IsTitled = true;
                                            ri.Position = MeetingPosition.StartingMeeting;
                                        }
                                        //Est-ce une fin de rendez-vous?
                                        else if (TempIdx + 1 >= GetIndex(EndDate))
                                            ri.Position = MeetingPosition.EndingMeeting;

                                        else //C'est dans un rendez-vous
                                            ri.Position = MeetingPosition.Meeting;

                                        ri.BorderPen = new Pen(vFirstMeetingFadeColor);
                                    }
                                    //Ce n'est pas un rendez-vous?
                                    else
                                    {
                                        ri.TextFont = vNonRdvTextFont;
                                        ri.TextColor = vRdvTextColor;
                                        ri.BackColor = vNonRdvBackColor;
                                        ri.Position = MeetingPosition.OutOfMeeting;
                                        //ri.BorderPen = new Pen(vNonRdvBackColor);                                        
                                    }
                                    ListToDraw.Add(ri);
                                }
                            }
                            i = GetIndex(EndDate) - 1;
                        }
                        #endregion

                        #region Une seule colonne
                        else
                        {
                            int one = GetIndex(vList[MeetingIdx].StartDate);

                            bool IsTitled = false;

                            for (; one < GetIndex(vList[MeetingIdx].EndDate); one++)
                            {
                                DayRectangleInfo ri = new DayRectangleInfo();
                                ri.TextFont = vRdvTextFont;
                                ri.TextColor = vRdvTextColor;
                                ri.BackColor = vFirstMeetingFadeColor;
                                ri.BackColor2 = vSecondMeetingFadeColor;
                                ri.Rectangle = new Rectangle(new Point(COLUMN_WIDTH, one * HEADER_HEIGHT),
                                    new Size(vSizeOfComponent.Width - vColumns, HEADER_HEIGHT));
                                ri.IDMeeting = vList[MeetingIdx].IDMeeting;

                                //Est-ce le dbut d'un rendez-vous
                                if (!IsTitled)
                                {
                                    ri.Text = vList[MeetingIdx].Title;
                                    ri.Position = MeetingPosition.StartingMeeting;
                                    IsTitled = true;
                                }

                                //Est-ce la fin d'un rendez-vous
                                else if (one + 1 >= GetIndex(vList[MeetingIdx].EndDate))
                                    ri.Position = MeetingPosition.EndingMeeting;

                                else //C'est dans un rendez-vous
                                    ri.Position = MeetingPosition.Meeting;

                                ri.BorderPen = new Pen(vFirstMeetingFadeColor);
                                ListToDraw.Add(ri);
                            }
                            i = one - 1;
                            MeetingIdx++;
                        }
                        #endregion
                    }
                    #endregion

                    #region Ce n'est pas un rendez-vous
                    else
                    {
                        DayRectangleInfo ri = new DayRectangleInfo();
                        ri.BackColor = vNonRdvBackColor;
                        ri.TextColor = vNonRdvTextColor;
                        ri.TextFont = vNonRdvTextFont;
                        ri.Text = "";
                        ri.Position = MeetingPosition.OutOfMeeting;

                        ri.Rectangle = new Rectangle(new Point(COLUMN_WIDTH, i * HEADER_HEIGHT),
                            new Size(vSizeOfComponent.Width - COLUMN_WIDTH, HEADER_HEIGHT));

                        ListToDraw.Add(ri);
                    }
                    #endregion
                }
            }
            #endregion

            #region Il n'y a aucun rendez-vous aujourd'hui
            else
            {
                for (int i = 0; i < DAYS_PARTS; i++)
                {
                    DayRectangleInfo ri = new DayRectangleInfo();
                    ri.BackColor = vNonRdvBackColor;
                    ri.TextColor = vNonRdvTextColor;
                    ri.TextFont = vNonRdvTextFont;
                    ri.Position = MeetingPosition.OutOfMeeting;

                    ri.Rectangle = new Rectangle(new Point(FirstStep + COLUMN_WIDTH, i * HEADER_HEIGHT),
                        new Size(vSizeOfComponent.Width, HEADER_HEIGHT));
                    ListToDraw.Add(ri);
                }
            }
            #endregion

            ListToDraw.AddRange(SetHeaders());
            vListToDraw = ListToDraw;
            return ListToDraw;
        }

        /// <summary>
        /// Retourne une SizeF donnant le position pour mettre un
        /// </summary>
        /// <param name="_ri">Le DayRectangleInfo contenant toutes les informations</param>
        /// <param name="_g">Graphics dans lequel on fait les mesures</param>
        /// <returns>Un point contenant les coordonnes du texte  afficher</returns>
        public Point GetMiddlePosition(DayRectangleInfo _ri, Graphics _g)
        {
            SizeF CoordString = _g.MeasureString(_ri.Text, _ri.TextFont);

            CoordString.Width = ((_ri.Rectangle.Width) / 2) - (CoordString.Width / 2);
            CoordString.Height = ((_ri.Rectangle.Height) / 2) - (CoordString.Height / 2);

            return new Point((int)CoordString.Width + _ri.Rectangle.X, (int)CoordString.Height + _ri.Rectangle.Y);
        }

        /// <summary>
        /// Donne la date en rapport avec l'index de sa position dans le tableau
        /// </summary>
        /// <param name="_index">Position de l'lment duquel il faut dduire la date</param>
        /// <returns>La date de l'lment</returns>
        public DateTime GetDate(int _index)
        {
            int div = 60 / (DAYS_PARTS / 24);

            int minute = _index * div;
            int hour = minute / 60;
            minute = minute % 60;

            if (hour == 24) hour = 0;

            return new DateTime(vDate.Year, vDate.Month, vDate.Day, hour, minute, 0);
        }
        #endregion

        #region Mthodes prives
        /// <summary>
        /// Initialise le header avec les valeurs par dfaut
        /// </summary>        
        protected DayRectangleInfo[] SetHeaders()
        {
            string h = "";
            int Col = GetColumnsNumber();
            string sHeure;
            string sMin;

            for (int i = 0; i < vHourHeader.Length; i++)
            {
                int heure = (int)(i / (DAYS_PARTS / 24));
                sHeure = "" + heure;
                int min = (i % (DAYS_PARTS / 24)) * (60 / (DAYS_PARTS / 24));
                sMin = "" + min;

                if (heure < 10)
                    sHeure = "0" + heure;
                if (min == 0)
                    sMin = "00";
                h = sHeure + "h" + sMin;

                #region Mise du texte et Affichage
                if (i < DAYS_PARTS)
                {
                    if (vHourHeader[i] == null)
                        vHourHeader[i] = new DayRectangleInfo();

                    vHourHeader[i].Text = h;
                    if (vHourHeader[i].BorderPen == null)
                        vHourHeader[i].BorderPen = new Pen(Color.Black);
                    if (vHourHeader[i].TextFont == null)
                        vHourHeader[i].TextFont = new Font("Times New Roman", 12);

                    vHourHeader[i].Rectangle =
                        new Rectangle(new Point(0, i * HEADER_HEIGHT),
                        new Size(COLUMN_WIDTH, HEADER_HEIGHT));
                }
                #endregion
            }
            return vHourHeader;
        }

        /// <summary>
        /// Donne l'index dans le tableau en rapport  l'heure donne
        /// </summary>
        /// <param name="_dt">L'heure pour laquelle il faut donner l'index</param>
        /// <returns>L'index dans le tableau des rendez-vous</returns>
        protected int GetIndex(DateTime _dt)
        {
            int min = 60 / (DAYS_PARTS / 24);
            int ret = _dt.Hour * (DAYS_PARTS / 24);
            ret += (int)(_dt.Minute / min);
            // ret--;
            return ret;
        }

        /// <summary>
        /// Prpare la liste des rendez-vous
        /// </summary>
        /// <returns>Le nombre de colonnes pour l'affichage</returns>
        protected int GetColumnsNumber()
        {
            int Col = 1;

            for (int i = 0; i < vList.Count - 1; i++)
            {
                if (vList[i].StartDate <= vList[i + 1].StartDate && vList[i].EndDate >= vList[i + 1].StartDate
                    || vList[i].StartDate <= vList[i + 1].EndDate && vList[i].EndDate >= vList[i + 1].EndDate)
                {
                    Col++;
                }
            }
            return Col;
        }

        /// <summary>
        /// Trouve l'index ou se situe l'lment du rendez-vous
        /// </summary>
        /// <param name="_Hour">Heure du rendez-vous</param>
        /// <returns>L'index dans le tableau</returns>
        protected int IndexOf(DateTime _Hour)
        {
            int hour = _Hour.Hour;
            int min = _Hour.Minute;

            return (hour * 4) + (min / 15);
        }

        /// <summary>
        /// Retourne les initiales du noms entr en argument
        /// </summary>
        /// <param name="_Name">Nom duquel il faut extraire les initiales</param>
        /// <returns>Les initiales du nom</returns>
        protected string GetInitials(string _Name)
        {
            string[] buf = _Name.Split(' ', '-');
            string ret = "";

            foreach (string s in buf)
            {
                string d = s.ToUpper();
                ret += d[0];
            }
            return ret;
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne la hauteur du composant
        /// </summary>
        public int Height
        {
            get { return vHourHeader.Length * HEADER_HEIGHT; }
        }
        /// <summary>
        /// Donne la hauteur d'une entre
        /// </summary>
        public int HeaderHeight { get { return HEADER_HEIGHT; } }
        /// <summary>
        /// Retourne ou initialise la liste des rendez-vous de la journe
        /// </summary>
        public List<MeetingRow> MeetingList
        {
            get { return vList; }
            set
            {
                if (value != null)
                {
                    //vList = value;
                    #region Tri des dates pour afficher seulement la date donne
                    List<MeetingRow> buf = new List<MeetingRow>();
                    foreach (MeetingRow mr in value)
                    {
                        if (mr.StartDate.Year == vDate.Year && mr.StartDate.Month == vDate.Month
                            && mr.StartDate.Day == vDate.Day)
                        {
                            buf.Add(mr);
                        }
                    }
                    Utils.Misc.Sort.BubbleSort<MeetingRow>(buf);
                    vList = buf;
                    this.Recalculate();
                    #endregion
                }
            }
        }
        /// <summary>
        /// Initialise la taille du composant dans lequel on dessine
        /// </summary>
        public Size SizeOfComponent
        {
            set { vSizeOfComponent = value; }
            get { return vSizeOfComponent; }
        }
        /// <summary>
        /// Retourne ou initialise la date  afficher
        /// </summary>
        public DateTime DateToDisplay
        {
            get { return vDate; }
            set { vDate = value; }
        }
        /// <summary>
        /// Retourne le nombre de cases contenues dans un jour
        /// </summary>
        public int Length { get { return DAYS_PARTS * this.GetColumnsNumber(); } }
        /// <summary>
        /// Retourne la liste des rectangle  dessiner
        /// </summary>
        public List<DayRectangleInfo> ListToDraw
        {
            get { return vListToDraw; }
            protected set { vListToDraw = value; }
        }
        /// <summary>
        /// Indique le nombre de parties que contiennent un jour
        /// </summary>
        public int DayParts { get { return DAYS_PARTS; } }
        /// <summary>
        /// Retourne la liste des headers
        /// </summary>
        public DayRectangleInfo[] Header { get { return vHourHeader; } }
        /// <summary>
        /// Retourne la liste des heures prises
        /// </summary>
        public List<MeetingRow> Hour { get { return vList; } }
        /// <summary>
        /// Retourne la largeur du header contenant les heures de la journe
        /// </summary>
        public int HeaderWidth { get { return COLUMN_WIDTH; } }
        /// <summary>
        /// Retourne le nombre de colonnes contenue dans le jour courrant
        /// </summary>
        public int Columns { get { return vColumns; } }
        #endregion

        #region Colors Options
        /// <summary>
        /// Retourne ou initialise la couleur de fond d'une heure sans rendez-vous
        /// </summary>
        public Color NonMeetingBackColor
        {
            get { return vNonRdvBackColor; }
            set
            {
                vNonRdvBackColor = value;
                this.Recalculate();
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur de fond des heures avec rendez-vous
        /// </summary>
        public Color FirstMeetingFadeColor
        {
            get { return vFirstMeetingFadeColor; }
            set
            {
                vFirstMeetingFadeColor = value;
                this.Recalculate();
            }
        }
        /// <summary>
        /// Retourne ou initialise la deuxime couleur de fond pour l'affichage d'un rendez-vous
        /// </summary>
        public Color SecondMeetingBackcolor
        {
            get { return this.vSecondMeetingFadeColor; }
            set { vSecondMeetingFadeColor = value; }
        }
        /// <summary>
        /// Retourne ou initialise la couleur du texte des heures  rendez-vous
        /// </summary>
        public Color RdvTextColor
        {
            get { return vRdvTextColor; }
            set
            {
                vRdvTextColor = value;
                this.Recalculate();
            }
        }
        /// <summary>
        /// Retourne ou initialise la police d'criture des heures avec rendez-vous
        /// </summary>
        public Font RdvTextFont
        {
            get { return vRdvTextFont; }
            set
            {
                vRdvTextFont = value;
                this.Recalculate();
            }
        }
        #endregion
    }

    /// <summary>
    /// Version de la classe Hours avec un buffer Image pour l'optimisation de l'affichage
    /// </summary>
    public class ImgHours : Hours
    {
        #region Variables
        /// <summary>
        /// Image buffer contenant l'image  dessiner
        /// </summary>
        protected Bitmap vImage;
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur par dfaut
        /// </summary>
        public ImgHours() : base() { }

        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_List">Initialise la liste de rendez-vous de la journe</param>
        public ImgHours(List<MeetingRow> _List) : base(_List) { }
        #endregion

        #region Mthodes
        /// <summary>
        /// Prpare les rectangles dans une image pour un dessinage ultrieur
        /// </summary>        
        public void Draw()
        {
            vImage = new Bitmap(base.SizeOfComponent.Width, base.Height);
            Graphics g = Graphics.FromImage(vImage);

            foreach (DayRectangleInfo ri in base.Recalculate())
            {
                #region Dessinage
                LinearGradientBrush lgb =
                    new LinearGradientBrush(ri.Rectangle, ri.BackColor, ri.BackColor2, LinearGradientMode.Horizontal);
                g.FillRectangle(lgb, ri.Rectangle);


                switch (ri.Position)
                {
                    case MeetingPosition.StartingMeeting:
                        #region StartingMeeting
                        //Ligne suprieure
                        g.DrawLine(new Pen(Color.Black), ri.Rectangle.Location,
                            new Point(ri.Rectangle.Location.X + SizeOfComponent.Width, ri.Rectangle.Location.Y));

                        g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                        break;
                        #endregion

                    case MeetingPosition.Meeting:
                        #region Meeting

                        break;
                        #endregion

                    case MeetingPosition.EndingMeeting:
                        #region EndingMeeting
                        //Ligne infrieure
                        g.DrawLine(new Pen(Color.Black), new Point(ri.Rectangle.X, ri.Rectangle.Y + base.HeaderHeight),
                            new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + base.HeaderHeight));
                        break;
                        #endregion

                    case MeetingPosition.OutOfMeeting:
                        #region OutOfMeeting
                        g.FillRectangle(new SolidBrush(ri.BackColor), ri.Rectangle);

                        //Ligne suprieure
                        g.DrawLine(ri.BorderPen, ri.Rectangle.Location,
                            new Point(ri.Rectangle.Location.X + ri.Rectangle.Width, ri.Rectangle.Location.Y));

                        //Ligne infrieure
                        g.DrawLine(ri.BorderPen, new Point(ri.Rectangle.X, ri.Rectangle.Y + base.HeaderHeight),
                            new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + base.HeaderHeight));

                        g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                        break;
                        #endregion
                }
                #endregion
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne une image correspondant  l'affichage du jour
        /// </summary>
        public Bitmap Image
        {
            get { return vImage; }
            protected set { vImage = value; }
        }

        /// <summary>
        /// Retourne ou initialise la taille du composant parent dans lequel on dessine le jour
        /// </summary>
        public new Size SizeOfComponent
        {
            get { return base.SizeOfComponent; }
            set
            {
                base.SizeOfComponent = value;
                this.Draw();
            }
        }

        /// <summary>
        /// Retourne ou initialise la liste des rendez-vous de la journe
        /// </summary>
        public new List<MeetingRow> MeetingList
        {
            get { return base.vList; }
            set
            {
                base.MeetingList = value;
                this.Draw();
            }
        }
        #endregion
    }
    #endregion

    #region Struct
    /// <summary>
    /// Structure de base pour le dessin d'une case du MonthAgenda ou du DayAgenda
    /// </summary>
    public class RectangleInfo
    {
        /// <summary>
        /// Constructeur
        /// </summary>
        public RectangleInfo()
        {
            BackColor = Color.White;
            Rectangle = new Rectangle();
            TextFont = new Font("Times new Roman", 12);
            TextColor = Color.Black;
            BorderPen = new Pen(Color.Silver);
            Text = "";
        }
        /// <summary>
        /// Taille de la case
        /// </summary>
        public Rectangle Rectangle;
        /// <summary>
        /// Texte  afficher dans la case
        /// </summary>
        public string Text;
        /// <summary>
        /// Police du texte
        /// </summary>
        public Font TextFont;
        /// <summary>
        /// Couleur du texte
        /// </summary>
        public Color TextColor;
        /// <summary>
        /// Couleur de la case
        /// </summary>
        public Color BackColor;
        /// <summary>
        /// Couleur de contour du rectangle
        /// </summary>
        public Pen BorderPen;
    }

    /// <summary>
    /// Informations d'une case du MonthAgenda
    /// </summary>
    /// <remarks>Utilisation de la classe comme une enum avec contstructeur par dfaut</remarks>
    public class MonthRectangleInfo : RectangleInfo
    {
        /// <summary>
        /// Constructeur
        /// </summary>
        public MonthRectangleInfo() : base() { DayNumber = ""; }
        /// <summary>
        /// Case avant aprs ou dans le mois courrant
        /// </summary>
        public CasePositionInMonth Position;
        /// <summary>
        /// Numro du jour  afficher
        /// </summary>
        public string DayNumber;
    }

    /// <summary>
    /// Informations d'une case du DayhAgenda
    /// </summary>
    /// <remarks>Utilisation de la classe comme une enum avec contstructeur par dfaut</remarks>
    public class DayRectangleInfo : RectangleInfo
    {
        /// <summary>
        /// Indique la position de la case dans le rendez-vous
        /// </summary>
        public MeetingPosition Position;
        /// <summary>
        /// Deuxime couleur pour le dgrad
        /// </summary>
        public Color BackColor2;
        /// <summary>
        /// Identifiant du rendez-vous
        /// </summary>
        public int IDMeeting;
    }

    /// <summary>
    /// Informations d'une case du HourSelector
    /// </summary>
    /// <remarks>Utilisation de la classe comme une enum avec contstructeur par dfaut</remarks>
    public class HourRectangleInfo : RectangleInfo
    {
        /// <summary>
        /// Constructeur
        /// </summary>
        public HourRectangleInfo() : base() { BackColor2 = Color.Silver; }
        /// <summary>
        /// Deuxime couleur de fond pour l'alternance de couleur
        /// </summary>
        public Color BackColor2;
    }

    /// <summary>
    /// Enumration qui donne la position de la case dans le mois
    /// </summary>
    public enum CasePositionInMonth
    {
        /// <summary>
        /// La case se trouve avant le mois
        /// </summary>
        Before,
        /// <summary>
        /// La case est dans le mois
        /// </summary>
        InMonth,
        /// <summary>
        /// La case est aprs le mois
        /// </summary>
        After,
    }

    /// <summary>
    /// Indique la position de la case dans le rendez-vous
    /// </summary>
    public enum MeetingPosition
    {
        /// <summary>
        /// Case hors du rendez-vous
        /// </summary>
        OutOfMeeting,
        /// <summary>
        /// Premire case d'un rendez-vous
        /// </summary>
        StartingMeeting,
        /// <summary>
        /// Est dans un rendez-vous
        /// </summary>
        Meeting,
        /// <summary>
        /// Est le fin d'un rendez-vous
        /// </summary>
        EndingMeeting,
    }

    /// <summary>
    /// Donne l'affichage des minutes dans le HourSelector
    /// </summary>
    public enum MinutesSelection
    {
        /// <summary>
        /// Affiche par minutes
        /// </summary>
        Minute,
        /// <summary>
        /// Affiche par 5 minutes
        /// </summary>
        FiveMinutes,
        /// <summary>
        /// Affiche par 1/4 d'heure
        /// </summary>
        Quarter,
        /// <summary>
        /// Affiche par demi heure
        /// </summary>
        HalfAnHour,
    }
    #endregion
}
